@@ -476,14 +476,20 @@ class Scene(object):
             self_bands, bands, scaling, data_type
         )
 
+        mask_nodata = bool(mask_nodata)
+
         alpha_band_name = "alpha"
         if isinstance(mask_alpha, six.string_types):
             alpha_band_name = mask_alpha
+            mask_alpha = True
         elif mask_alpha is None:
             # if user does not set mask_alpha, only attempt to mask_alpha if
             # alpha band is exists in the scene.
             mask_alpha = self.has_alpha(alpha_band_name)
+        elif type(mask_alpha) is not bool:
+            raise ValueError("'mask_alpha' must be None, a band name, or a bool.")
 
+        drop_alpha = False
         if mask_alpha:
             if not self.has_alpha(alpha_band_name):
                 raise ValueError(
@@ -502,7 +508,6 @@ class Scene(object):
                     raise ValueError(
                         "Alpha must be the last band in order to reduce rasterization errors"
                     )
-                drop_alpha = False
 
         raster_params = ctx.raster_params
         full_raster_args = dict(
@@ -513,6 +518,10 @@ class Scene(object):
             data_type=dtype,
             resampler=resampler,
             processing_level=processing_level,
+            masked=mask_nodata or mask_alpha,
+            mask_nodata=mask_nodata,
+            mask_alpha=mask_alpha,
+            drop_alpha=drop_alpha,
             **raster_params
         )
 
@@ -542,16 +551,6 @@ class Scene(object):
             # if only 1 band requested, still return a 3d array
             arr = arr[np.newaxis]
 
-        if mask_nodata or mask_alpha:
-            if mask_alpha:
-                alpha = arr[-1]
-                if drop_alpha:
-                    arr = arr[:-1]
-                    bands.pop(-1)
-                arr.mask = ~alpha.astype(bool)
-        else:
-            arr = arr.data
-
         if bands_axis != 0:
             arr = np.moveaxis(arr, 0, bands_axis)
         if raster_info:
@@ -575,7 +574,7 @@ class Scene(object):
         raster_client=None,
     ):
         """
-        Save bands from this scene as a GeoTIFF, PNG, or JPEG, writing to a path or file-like object.
+        Save bands from this scene as a GeoTIFF, PNG, or JPEG, writing to a path.
 
         Parameters
         ----------
@@ -586,7 +585,7 @@ class Scene(object):
             Names must be keys in ``self.properties.bands``.
         ctx : :class:`~descarteslabs.scenes.geocontext.GeoContext`
             A :class:`~descarteslabs.scenes.geocontext.GeoContext` to use when loading this Scene
-        dest : str, path-like object, or file-like object, default None
+        dest : str or path-like object, default None
             Where to write the image file.
 
             * If None (default), it's written to an image file of the given ``format``
@@ -600,9 +599,8 @@ class Scene(object):
 
               Note that path-like objects (such as pathlib.Path) are only supported
               in Python 3.6 or later.
-            * If a file-like object, it's written into that file.
         format : str, default "tif"
-            If a file-like object or None is given as ``dest``: one of "tif", "png", or "jpg".
+            If None is given as ``dest``: one of "tif", "png", or "jpg".
 
             If a str or path-like object is given as ``dest``, ``format`` is ignored
             and determined from the extension on the path (one of ".tif", ".png", or ".jpg").
@@ -647,8 +645,6 @@ class Scene(object):
         ...     "rasters/{ctx.resolution}-{scene.properties.id}.jpg".format(ctx=ctx, scene=scene)
         ... )  # doctest: +SKIP
         "rasters/15-landsat:LC08:PRE:TOAR:meta_LC80270312016188_v1.tif"
-        >>> with open("another_raster.jpg", "wb") as f:
-        ...     scene.download("swir2", ctx, dest=f, format="jpg")  # doctest: +SKIP
 
         Raises
         ------
